@@ -2,6 +2,8 @@
 
 use crate::structures::gdt::SegmentSelector;
 use crate::VirtAddr;
+#[cfg(feature = "inline_asm")]
+use core::arch::asm;
 
 pub use crate::structures::DescriptorTablePointer;
 
@@ -19,10 +21,14 @@ pub use crate::structures::DescriptorTablePointer;
 #[inline]
 pub unsafe fn lgdt(gdt: &DescriptorTablePointer) {
     #[cfg(feature = "inline_asm")]
-    asm!("lgdt [{}]", in(reg) gdt, options(readonly, nostack, preserves_flags));
+    unsafe {
+        asm!("lgdt [{}]", in(reg) gdt, options(readonly, nostack, preserves_flags));
+    }
 
     #[cfg(not(feature = "inline_asm"))]
-    crate::asm::x86_64_asm_lgdt(gdt as *const _);
+    unsafe {
+        crate::asm::x86_64_asm_lgdt(gdt as *const _);
+    }
 }
 
 /// Load an IDT.
@@ -39,10 +45,31 @@ pub unsafe fn lgdt(gdt: &DescriptorTablePointer) {
 #[inline]
 pub unsafe fn lidt(idt: &DescriptorTablePointer) {
     #[cfg(feature = "inline_asm")]
-    asm!("lidt [{}]", in(reg) idt, options(readonly, nostack, preserves_flags));
+    unsafe {
+        asm!("lidt [{}]", in(reg) idt, options(readonly, nostack, preserves_flags));
+    }
 
     #[cfg(not(feature = "inline_asm"))]
-    crate::asm::x86_64_asm_lidt(idt as *const _);
+    unsafe {
+        crate::asm::x86_64_asm_lidt(idt as *const _);
+    }
+}
+
+/// Get the address of the current GDT.
+#[inline]
+pub fn sgdt() -> DescriptorTablePointer {
+    let mut gdt: DescriptorTablePointer = DescriptorTablePointer {
+        limit: 0,
+        base: VirtAddr::new(0),
+    };
+    unsafe {
+        #[cfg(feature = "inline_asm")]
+        asm!("sgdt [{}]", in(reg) &mut gdt, options(nostack, preserves_flags));
+
+        #[cfg(not(feature = "inline_asm"))]
+        crate::asm::x86_64_asm_sgdt(&mut gdt as *mut _);
+    }
+    gdt
 }
 
 /// Get the address of the current IDT.
@@ -72,8 +99,12 @@ pub fn sidt() -> DescriptorTablePointer {
 #[inline]
 pub unsafe fn load_tss(sel: SegmentSelector) {
     #[cfg(feature = "inline_asm")]
-    asm!("ltr {0:x}", in(reg) sel.0, options(nomem, nostack, preserves_flags));
+    unsafe {
+        asm!("ltr {0:x}", in(reg) sel.0, options(nomem, nostack, preserves_flags));
+    }
 
     #[cfg(not(feature = "inline_asm"))]
-    crate::asm::x86_64_asm_ltr(sel.0);
+    unsafe {
+        crate::asm::x86_64_asm_ltr(sel.0);
+    }
 }
